/* Linux driver for Philips webcam 
   USB and Video4Linux interface part.
   (C) 1999-2002 Nemosoft Unv.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*  
   This code forms the interface between the USB layers and the Philips
   specific stuff. Some adanved stuff of the driver falls under an
   NDA, signed between me and Philips B.V., Eindhoven, the Netherlands, and
   is thus not distributed in source form. The binary pwcx.o module 
   contains the code that falls under the NDA.
   
   In case you're wondering: 'pwc' stands for "Philips WebCam", but 
   I really didn't want to type 'philips_web_cam' every time (I'm lazy as
   any Linux kernel hacker, but I don't like uncomprehensible abbreviations
   without explanation).
   
   Oh yes, convention: to disctinguish between all the various pointers to
   device-structures, I use these names for the pointer variables:
   udev: struct usb_device *
   vdev: struct video_device *
   pdev: struct wct_devive *
*/

/* Contributors:
   - Alvarado: adding whitebalance code
   - Alistar Moire: QuickCam 3000 Pro device/product ID
   - Tony Hoyle: Creative Labs Webcam 5 device/product ID
   - Mark Burazin: solving hang in VIDIOCSYNC when camera gets unplugged
   - Jk Fang: SOTEC device/product ID
*/

/*
 Changes:

  This is now part of the wctweak_lib project to build a static Win32 library

  Additions and changes and mostly deletions for a Win32 Version by BM 20031112
  This is only to support the IOCTLs for the cameras to fiddle  with some features 
  not available in the regular driver all the video stuff has been deleted.

  USB communication takes place through the libusb-win32 drivers

  All restrictions and applications mentioned above still apply to this file
   -- wctweak_lib project (C)2003 M. Burri 

  have deleted most of the code - otherwise it was to crowded with things not to 
  bother with....

  I've renamed the public interface from pwc_ to wct_ as it does not provide
  the full wet of pwc 8.6 and shoul not be mixed up with it.

  Thank you guys for the great work done.
*/

#include <stdio.h>
#include <malloc.h>
#include <memory.h>

#include <include\usb.h>  //BM 

#include "wct.h"


/* Function prototypes and driver templates */

#if 0
/* kown pairs */
static __devinitdata struct usb_device_id wct_device_table [] = {
	{ USB_DEVICE(0x0471, 0x0302) }, /* Philips models */
	{ USB_DEVICE(0x0471, 0x0303) },
	{ USB_DEVICE(0x0471, 0x0304) },
	{ USB_DEVICE(0x0471, 0x0307) },
	{ USB_DEVICE(0x0471, 0x0308) },
	{ USB_DEVICE(0x0471, 0x030C) },
	{ USB_DEVICE(0x0471, 0x0310) },
	{ USB_DEVICE(0x0471, 0x0311) },
	{ USB_DEVICE(0x0471, 0x0312) },
	{ USB_DEVICE(0x0471, 0x0313) }, /* the 'new' 720K */
	{ USB_DEVICE(0x069A, 0x0001) }, /* Askey */
	{ USB_DEVICE(0x046D, 0x08B0) }, /* Logitech QuickCam Pro 3000 */
	{ USB_DEVICE(0x046D, 0x08B1) }, /* Logitech QuickCam Notebook Pro */
	{ USB_DEVICE(0x046D, 0x08B2) }, /* Logitech QuickCam Pro 4000 */
	{ USB_DEVICE(0x046D, 0x08B3) }, /* Logitech QuickCam Zoom (old model) */
	{ USB_DEVICE(0x046D, 0x08B4) }, /* Logitech QuickCam Zoom (new model) */
	{ USB_DEVICE(0x046D, 0x08B5) }, /* Logitech QuickCam Orbit/Sphere */
	{ USB_DEVICE(0x046D, 0x08B6) }, /* Logitech (reserved) */
	{ USB_DEVICE(0x046D, 0x08B7) }, /* Logitech (reserved) */
	{ USB_DEVICE(0x046D, 0x08B8) }, /* Logitech (reserved) */
	{ USB_DEVICE(0x055D, 0x9000) }, /* Samsung */
	{ USB_DEVICE(0x055D, 0x9001) },
	{ USB_DEVICE(0x041E, 0x400C) }, /* Creative Webcam 5 */
	{ USB_DEVICE(0x041E, 0x4011) }, /* Creative Webcam Pro Ex */
	{ USB_DEVICE(0x04CC, 0x8116) }, /* Afina Eye */
	{ USB_DEVICE(0x0d81, 0x1910) }, /* Visionite */
	{ USB_DEVICE(0x0d81, 0x1900) },
	{ }
};
#endif // omit

#define MAX_DEV_HINTS 10

static int power_save = 0;
static int led_on = 100, led_off = 0; /* defaults to LED that is on while in use */
static struct {
	int type;
	char serial_number[30];
	int device_node;
	struct wct_device *pdev;
} device_hint[MAX_DEV_HINTS];


int wct_dbg_flg = 0;

static void zz_devProps(int vendor_id, int product_id, int *type_id, char *name)
{

	if (vendor_id == 0x0471) {
		switch (product_id) {
		case 0x0302:
			sprintf(name, "Philips PCA645VC USB webcam detected.\n");
			*type_id = 645;
			break;
		case 0x0303:
			sprintf(name, "Philips PCA646VC USB webcam detected.\n");
			*type_id = 646;
			break;
		case 0x0304:
			sprintf(name, "Askey VC010 type 2 USB webcam detected.\n");
			*type_id = 646;
			break;
		case 0x0307:
			sprintf(name, "Philips PCVC675K (Vesta) USB webcam detected.\n");
			*type_id = 675;
			break;
		case 0x0308:
			sprintf(name, "Philips PCVC680K (Vesta Pro) USB webcam detected.\n");
			*type_id = 680;
			break;
		case 0x030C:
			sprintf(name, "Philips PCVC690K (Vesta Pro Scan) USB webcam detected.\n");
			*type_id = 690;
			break;
		case 0x0310:
			sprintf(name, "Philips PCVC730K (ToUCam Fun) USB webcam detected.\n");
			*type_id = 730;
			break;
		case 0x0311:
			sprintf(name, "Philips PCVC740K (ToUCam Pro) USB webcam detected.\n");
			*type_id = 740;
			break;
		case 0x0312:
			sprintf(name, "Philips PCVC750K (ToUCam Pro Scan) USB webcam detected.\n");
			*type_id = 750;
			break;
		case 0x0313:
			sprintf(name, "Philips PCVC720K/40 (ToUCam XS) USB webcam detected.\n");
			*type_id = 720;
			break;
		default:
			sprintf(name, "Unknown Philips USB webcam detected.\n");
			*type_id = -1;
			break;
		}
	}
	else if (vendor_id == 0x069A) {
		switch(product_id) {
		case 0x0001:
			sprintf(name, "Askey VC010 type 1 USB webcam detected.\n");
			*type_id = 645;
			break;
		default:
			sprintf(name, "Unknown Askey USB webcam detected.\n");
			*type_id = -1;
			break;
		}
	}
	else if (vendor_id == 0x046d) {
		switch(product_id) {
		case 0x08b0:
			sprintf(name, "Logitech QuickCam Pro 3000 USB webcam detected.\n");
			*type_id = 740; /* CCD sensor */
			break;
		case 0x08b1:
			sprintf(name, "Logitech QuickCam Notebook Pro USB webcam detected.\n");
			*type_id = 740; /* CCD sensor */
			break;
		case 0x08b2:
			sprintf(name, "Logitech QuickCam 4000 Pro USB webcam detected.\n");
			*type_id = 740; /* CCD sensor */
			break;
		case 0x08b3:
			sprintf(name, "Logitech QuickCam Zoom USB webcam detected.\n");
			*type_id = 740; /* CCD sensor */
			break;
		case 0x08B4:
			sprintf(name, "Logitech QuickCam Zoom (new model) USB webcam detected.\n");
			*type_id = 740; /* CCD sensor */
			break;
		case 0x08b5:
			sprintf(name, "Logitech QuickCam Orbit/Sphere USB webcam detected.\n");
			*type_id = 740; /* CCD sensor */
			break;
		case 0x08b6:
		case 0x08b7:
		case 0x08b8:
			sprintf(name, "Logitech QuickCam detected (reserved ID).\n");
			*type_id = 730; /* Assuming CMOS */
			break;
       	default:
			sprintf(name, "Unknown Logitech USB webcam detected.\n");
			*type_id = -1;
       		break;
        	}
        }
	else if (vendor_id == 0x055d) {
		/* I don't know the difference between the C10 and the C30;
		   I suppose the difference is the sensor, but both cameras
		   work equally well with a type_id of 675
		 */
		switch(product_id) {
		case 0x9000:
			sprintf(name, "Samsung MPC-C10 USB webcam detected.\n");
			*type_id = 675;
			break;
		case 0x9001:
			sprintf(name, "Samsung MPC-C30 USB webcam detected.\n");
			*type_id = 675;
			break;
		default:
			sprintf(name, "Unknown Samsung USB webcam detected.\n");
			*type_id = -1;
			break;
		}
	}
	else if (vendor_id == 0x041e) {
		switch(product_id) {
		case 0x400c:
			sprintf(name, "Creative Labs Webcam 5 detected.\n");
			*type_id = 730;
			break;
		case 0x4011:
			sprintf(name, "Creative Labs Webcam Pro Ex detected.\n");
			*type_id = 740;
			break;
		default:
			sprintf(name, "Unknown Creative Labs USB webcam detected.\n");
			*type_id = -1;
			break;
		}
	}
	else if (vendor_id == 0x04cc) { 
		switch(product_id) {
		case 0x8116:
			sprintf(name, "Sotec Afina Eye USB webcam detected.\n");
			*type_id = 730;
			break;  
		default:
			sprintf(name, "Unknown Sotec Afina Eye USB webcam detected.\n");
			*type_id = -1;
			break;
		}
	}
	else if (vendor_id == 0x0d81) {
		switch(product_id) {
		case 0x1900:
			sprintf(name, "Visionite VCS-UC300 USB webcam detected.\n");
			*type_id = 740; /* CCD sensor */
			break;
		case 0x1910:
			sprintf(name, "Visionite VCS-UM100 USB webcam detected.\n");
			*type_id = 730; /* CMOS sensor */
			break;
		default:
			sprintf(name, "Unknown Visionite Eye USB webcam detected.\n");
			*type_id = -1;
			break;
		}
	}
	else 
	{
		sprintf(name, "Unknown USB webcam detected.\n");
		*type_id = -1;
	}

}

/***************************************************************************/

/* Okay, this is some magic that I worked out and the reasoning behind it...

   The biggest problem with any USB device is of course: "what to do 
   when the user unplugs the device while it is in use by an application?"
   We have several options:
   1) Curse them with the 7 plagues when they do (requires divine intervention)
   2) Tell them not to (won't work: they'll do it anyway)
   3) Oops the kernel (this will have a negative effect on a user's uptime)
   4) Do something sensible.
   
   Of course, we go for option 4.

  Changes:
  well the wctweak_lib goes for option 2 as it will not be connected long enough
  to bother with unplugged cams

   It happens that this device will be linked to two times, once from
   usb_device and once from the video_device in their respective 'private'
   pointers. This is done when the device is probed() and all initialization
   succeeded. The wct_device struct links back to both structures.

   When a device is unplugged while in use it will be removed from the 
   list of known USB devices; I also de-register it as a V4L device, but 
   unfortunately I can't free the memory since the struct is still in use
   by the file descriptor. This free-ing is then deferend until the first
   opportunity. Crude, but it works.
   
   A small 'advantage' is that if a user unplugs the cam and plugs it back
   in, it should get assigned the same video device minor, but unfortunately
   it's non-trivial to re-link the cam back to the video device... (that 
   would surely be magic! :))
*/


/***************************************************************************/
/* Public functions */

WCT_HANDLE wct_create_handle() 
{

struct wct_device *pdev;

	pdev = malloc(sizeof(struct wct_device));
	memset(pdev, 0, sizeof(struct wct_device));
	pdev->initCheck = WCT_INIT_CONST; // really my memblock

	return (WCT_HANDLE)pdev;
}//wct_create_handle


void wct_free_handle(WCT_HANDLE wct) 
{
	if (_msize(wct)>0) 
	{
		free(wct);
	}
}//wct_create_handle



int wct_open(WCT_HANDLE wct, unsigned short vendor, unsigned short product)
{

struct wct_device *pdev	= (struct wct_device *)wct; // takeover

int i;
const char *sensor_type = NULL;
struct usb_bus *bus;
struct usb_bus *usb_busses;
struct usb_device *dev;

struct usb_device *udev = NULL;
int type_id;
int ret;
int video_nr = -1; /* default: use next available device */
char serial_number[30];
char strbuffer[256];




	if ( !pdev || (_msize(wct)==0) ) // paranoia..
	{
		return -1;
	}

	if (pdev->initCheck != WCT_INIT_CONST) // more paranoia..
	{
		return -1;
	}

	if (!pdev->usb_init) {
		// USB not yet connected

		pdev->udev = NULL;
		pdev->udevH = NULL;

		usb_init();
		pdev->usb_init = 1;
	}

	if (pdev->udevH)
	{
		usb_release_interface(pdev->udevH, 0);
		usb_close(pdev->udevH); pdev->udevH = NULL;
	}
	if (wct_dbg_flg) usb_set_debug(2);

	i = usb_find_busses();
	i = usb_find_devices();
	usb_busses = usb_get_busses();


	for (bus = usb_busses; bus; bus = bus->next) {
		for (dev = bus->devices; dev; dev = dev->next) {
			if ( (dev->descriptor.idVendor == vendor) && (dev->descriptor.idProduct==product) ) 
			{
				fprintf(stderr, "%s/%s     %04X/%04X\n", bus->dirname, dev->filename,
						dev->descriptor.idVendor, dev->descriptor.idProduct);
				pdev->udev = dev;
				pdev->udevH = usb_open(dev);
			}
  

		}
	}

	if (!pdev->udevH) 
	{
		fprintf(stderr, "  Couldn't retrieve usb device\n");
		return -1;
	}

	zz_devProps(pdev->udev->descriptor.idVendor, 
				pdev->udev->descriptor.idProduct,
				&type_id, strbuffer);
	fprintf(stderr, "%s\n", strbuffer);

	memset(serial_number, 0, 30);
    if (pdev->udev->descriptor.iSerialNumber) {
      ret = usb_get_string_simple(pdev->udevH, 
								pdev->udev->descriptor.iSerialNumber, serial_number, 29);
      if (ret > 0)
        fprintf(stderr, "- Serial Number: %s\n", serial_number);
      else
        fprintf(stderr, "- Unable to fetch serial number string\n");
    }


	if (pdev->udev->descriptor.bNumConfigurations > 1)
		fprintf(stderr, "Warning: more than 1 configuration available.\n");

	pdev->type = type_id;
	//wct_construct(pdev);

// not needed for this type of operation
/* Reset camera */
//	if (usb_claim_interface(pdev->udevH, 0))
//	{
//		fprintf(stderr, "  Failed to claim interface\n");
//		usb_close(pdev->udevH); pdev->udevH = NULL;
//		return -1;
//	}
		
	switch(pdev->type) {
	case 645:
	case 646:
		pdev->vcinterface = 2;
		pdev->vendpoint = 4;
		break;
	case 675:
	case 680:
	case 690:
		pdev->vcinterface = 3;
		pdev->vendpoint = 4;
		break;
	case 730:
	case 740:
	case 750:
		pdev->vcinterface = 3;
		pdev->vendpoint = 5;
		break;
	}
	/*
	if (usb_set_altinterface(pdev->udevH, pdev->vcinterface))
	{
		printf("  Failed to set alternate interface\n");
		usb_close(pdev->udevH); pdev->udevH = NULL;
		return -1;
	}
	*/
	/* Query CMOS sensor type */

	i = wct_get_cmos_sensor(pdev);
	switch(i) {
	case -1: /* Unknown, show nothing */; break;
	case 0x00:  sensor_type = "Hyundai CMOS sensor"; break;
	case 0x20:  sensor_type = "Sony CCD sensor + TDA8787"; break;
	case 0x2E:  sensor_type = "Sony CCD sensor + Exas 98L59"; break;
	case 0x2F:  sensor_type = "Sony CCD sensor + ADI 9804"; break;
	case 0x30:  sensor_type = "Sharp CCD sensor + TDA8787"; break;
	case 0x3E:  sensor_type = "Sharp CCD sensor + Exas 98L59"; break;
	case 0x3F:  sensor_type = "Sharp CCD sensor + ADI 9804"; break;
	case 0x40:  sensor_type = "UPA 1021 sensor"; break;
	case 0x100: sensor_type = "VGA sensor"; break;
	case 0x101: sensor_type = "PAL MR sensor"; break;
	default:   sensor_type = "unknown type of sensor"; break;
	}
	if (sensor_type != NULL)
		fprintf(stderr, "This camera is equipped with a %s (%d).\n", sensor_type, i);

	/* Turn on camera */
	if (power_save) 
	{
		i = wct_camera_power(pdev, 1);
		if (i < 0)
			fprintf(stderr, "Failed to restore power to the camera! (%d)\n", i);
	}
	/* Set LED on/off time */
	if (wct_set_leds(pdev, led_on, led_off) < 0)
		fprintf(stderr, "Failed to set LED on/off time.\n");


	/* So far, so good. Allocate memory. */
	/* Reset buffers & parameters */

	/* Set some defaults */

	/* Start iso pipe for video; first try user-supplied size/fps, if
	   that fails try QCIF/10 or QSIF/10 (a reasonable default), 
	   then give up 
	 */

	pdev->vopen++;
	/* lock decompressor; this has a small race condition, since we 
	   could in theory unload pwcx.o between wct_find_decompressor()
	   above and this call. I doubt it's ever going to be a problem.
	 */
	return 0;
}




/* Note that all cleanup is done in the reverse order as in _open */
void wct_close(WCT_HANDLE wct)
{
struct wct_device *pdev = (struct wct_device *)wct; // takeover

int i;


	if ( !pdev || (_msize(wct)==0) ) // paranoia..
	{
		return;
	}

	if (pdev->initCheck != WCT_INIT_CONST) // more paranoia..
	{
		return;
	}

	if (pdev->vopen >= 0)
	{

		/* Turn LEDs off */
		if (wct_set_leds(pdev, 0, 0) < 0)
			fprintf(stderr, "Failed to set LED on/off time..\n");
		/* Power down camera to save energy */
		if (power_save) 
			i = wct_camera_power(pdev, 0);
	}

	pdev->vopen = 0;

	if (pdev->udevH) 
	{
		usb_release_interface(pdev->udevH, 0);
		usb_close(pdev->udevH); 
		pdev->udevH = NULL;
	}
	pdev->usb_init = 0;
}//wct_close


char *wct_scanusb(WCT_HANDLE wct)
{
struct wct_device *pdev	= (struct wct_device *)wct; // takeover

struct usb_bus *bus;
struct usb_bus *usb_busses;
struct usb_device *dev;
usb_dev_handle *udev;
int i, ret, bufIdx=0, type_id;
char string[256];
char *buf;


	if ( !pdev || (_msize(wct)==0) ) // paranoia..
	{
		return NULL;
	}

	if (pdev->initCheck != WCT_INIT_CONST) // more paranoia..
	{
		return NULL;
	}

	if (!pdev->usb_init) {
		// USB not yet connected

		pdev->udev = NULL;
		pdev->udevH = NULL;

		usb_init();
		pdev->usb_init = 1;
	}
	if (wct_dbg_flg) usb_set_debug(10);

	i = usb_find_busses();
	i = usb_find_devices();
	usb_busses = usb_get_busses();

	buf = (char*)malloc(4096); // must be enough....
	bufIdx = 0; 
	if (!buf)
		return NULL;

	for (bus = usb_busses; bus; bus = bus->next) 
	{
		for (dev = bus->devices; dev; dev = dev->next) 
		{
			bufIdx += sprintf(buf+bufIdx, "\nUSB Device found:\n");
			bufIdx += sprintf(buf+bufIdx, " %s/%s     0x%04X,0x%04X\n", bus->dirname, dev->filename,
					dev->descriptor.idVendor, dev->descriptor.idProduct);

			udev = usb_open(dev);

			if (udev) 
			{
				if (dev->descriptor.iManufacturer) {
					ret = usb_get_string_simple(udev, dev->descriptor.iManufacturer, string, sizeof(string));
					if (ret > 0)
						bufIdx += sprintf(buf+bufIdx, "- Manufacturer : %s\n", string);
					else
						bufIdx += sprintf(buf+bufIdx, "- Unable to fetch manufacturer string\n");
				}

				if (dev->descriptor.iProduct) {
					ret = usb_get_string_simple(udev, dev->descriptor.iProduct, string, sizeof(string));
					if (ret > 0)
						bufIdx += sprintf(buf+bufIdx, "- Product      : %s\n", string);
					else
						bufIdx += sprintf(buf+bufIdx, "- Unable to fetch product string\n");
				}

				if (dev->descriptor.iSerialNumber) {
					ret = usb_get_string_simple(udev, dev->descriptor.iSerialNumber, string, sizeof(string));
					if (ret > 0)
						bufIdx += sprintf(buf+bufIdx, "- Serial Number: %s\n", string);
					else
						bufIdx += sprintf(buf+bufIdx, "- Unable to fetch serial number string\n");
				}

				usb_close (udev);
			}

			zz_devProps(dev->descriptor.idVendor, dev->descriptor.idProduct,
				&type_id, string);
				bufIdx += sprintf(buf+bufIdx, "%s\n", string);
		}
	}

	bufIdx += sprintf(buf+bufIdx, "\0");
	return buf;
}//wct_scanusb


/* *grunt* We have to do atoi ourselves :-( */
static int wct_atoi(char *s)
{
	int k = 0;
	
	k = 0;
	while (*s != '\0' && *s >= '0' && *s <= '9') {
		k = 10 * k + (*s - '0');
		s++;
	}
	return k;
}



