/* (C) 2001-2002 Nemosoft Unv.    webcam@smcc.demon.nl
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*         This is pwc-ioctl.h belonging to PWC 8.6                        */

/* 
   Changes
   2001/08/03  Alvarado   Added ioctl constants to access methods for 
                          changing white balance and red/blue gains
 */

/*
 Changes:

  This is now part of the wctweak_lib project to build a static Win32 library

  Additions and changes and mostly deletions for a Win32 Version by BM 20031112
  This is only to support the IOCTLs for the cameras to fiddle  with some features 
  not available in the regular driver all the video stuff has been deleted.

  USB communication takes place through the libusb-win32 drivers

  All restrictions and applications mentioned above still apply to this file
   -- wctweak_lib project (C)2003 M. Burri 

*/

/*
  This file pwc-ioctl.h remains private to the wctweak_lib build project
*/

#ifndef wct_IOCTL_H
#define wct_IOCTL_H


/* These are private ioctl() commands, specific for the Philips webcams.
   They contain functions not found in other webcams, and settings not
   specified in the Video4Linux API. 
   
   The #define names are built up like follows:
   VIDIOC		VIDeo IOCtl prefix
         PWC		Philps WebCam
            G           optional: Get
            S           optional: Set
             ... 	the function
 */

//BM included from Linux to minimize pwc changes required
// it does not support the generic IOCTL stuff anymore however

 #define _IOC_NRBITS     8
 #define _IOC_TYPEBITS   8
 #define _IOC_SIZEBITS   14
 #define _IOC_DIRBITS    2
 
 #define _IOC_NRMASK     ((1 << _IOC_NRBITS)-1)
 #define _IOC_TYPEMASK   ((1 << _IOC_TYPEBITS)-1)
 #define _IOC_SIZEMASK   ((1 << _IOC_SIZEBITS)-1)
 #define _IOC_DIRMASK    ((1 << _IOC_DIRBITS)-1)
 
 #define _IOC_NRSHIFT    0
 #define _IOC_TYPESHIFT  (_IOC_NRSHIFT+_IOC_NRBITS)
 #define _IOC_SIZESHIFT  (_IOC_TYPESHIFT+_IOC_TYPEBITS)
 #define _IOC_DIRSHIFT   (_IOC_SIZESHIFT+_IOC_SIZEBITS)
/*
   * Direction bits.
   */
 #define _IOC_NONE       0U
 #define _IOC_WRITE      1U
 #define _IOC_READ       2U
 
 #define _IOC(dir,type,nr,size) \
         (((dir)  << _IOC_DIRSHIFT) | \
          ((type) << _IOC_TYPESHIFT) | \
          ((nr)   << _IOC_NRSHIFT) | \
          ((size) << _IOC_SIZESHIFT))
#define _IO(type,nr)            _IOC(_IOC_NONE,(type),(nr),0)
#define _IOR(type,nr,size)      _IOC(_IOC_READ,(type),(nr),sizeof(size))
#define _IOW(type,nr,size)      _IOC(_IOC_WRITE,(type),(nr),sizeof(size))
#define _IOWR(type,nr,size)     _IOC(_IOC_READ|_IOC_WRITE,(type),(nr),sizeof(size))



/*
   * The following is for compatibility across the various Linux
   * platforms.  The i386 ioctl numbering scheme doesn't really enforce
   * a type field.  De facto, however, the top 8 bits of the lower 16
   * bits are indeed used as a type field, so we might just as well make
   * this explicit here.  Please be sure to use the decoding macros
   * below from now on.
   */

#define wct_FPS_SHIFT		16
#define wct_FPS_MASK		0x00FF0000
#define wct_FPS_FRMASK		0x003F0000
#define wct_FPS_SNAPSHOT	0x00400000



struct wct_probe
{
	char name[32];
	int type;
};





 /* Restore user settings */
#define VIDIOCPWCRUSER		_IO('v', 192)
 /* Save user settings */
#define VIDIOCPWCSUSER		_IO('v', 193)
 /* Restore factory settings */
#define VIDIOCPWCFACTORY	_IO('v', 194)

 /* You can manipulate the compression factor. A compression preference of 0
    means use uncompressed modes when available; 1 is low compression, 2 is
    medium and 3 is high compression preferred. Of course, the higher the
    compression, the lower the bandwidth used but more chance of artefacts
    in the image. The driver automatically chooses a higher compression when
    the preferred mode is not available.
  */
 /* Set preferred compression quality (0 = uncompressed, 3 = highest compression) */
#define VIDIOCPWCSCQUAL		_IOW('v', 195, int)
 /* Get preferred compression quality */
#define VIDIOCPWCGCQUAL		_IOR('v', 195, int)


 /* This is a probe function; since so many devices are supported, it
    becomes difficult to include all the names in programs that want to
    check for the enhanced Philips stuff. So in stead, try this PROBE;
    it returns a structure with the original name, and the corresponding 
    Philips type.
    To use, fill the structure with zeroes, call PROBE and if that succeeds,
    compare the name with that returned from VIDIOCGCAP; they should be the
    same. If so, you can be assured it is a Philips (OEM) cam and the type
    is valid.
 */    
#define VIDIOCPWCPROBE		_IOR('v', 199, struct wct_probe)

 /* Set AGC (Automatic Gain Control); int < 0 = auto, 0..65535 = fixed */
#define VIDIOCPWCSAGC		_IOW('v', 200, int)
 /* Get AGC; int < 0 = auto; >= 0 = fixed, range 0..65535 */
#define VIDIOCPWCGAGC		_IOR('v', 200, int)
 /* Set shutter speed; int < 0 = auto; >= 0 = fixed, range 0..65535 */
#define VIDIOCPWCSSHUTTER	_IOW('v', 201, int)

 /* Color compensation (Auto White Balance) */
#define VIDIOCPWCSAWB           _IOW('v', 202, struct wct_whitebalance)
#define VIDIOCPWCGAWB           _IOR('v', 202, struct wct_whitebalance)

 /* Auto WB speed */
#define VIDIOCPWCSAWBSPEED	_IOW('v', 203, struct wct_wb_speed)
#define VIDIOCPWCGAWBSPEED	_IOR('v', 203, struct wct_wb_speed)

 /* LEDs on/off/blink; int range 0..65535 */
#define VIDIOCPWCSLED           _IOW('v', 205, struct wct_leds)
#define VIDIOCPWCGLED           _IOR('v', 205, struct wct_leds)

  /* Contour (sharpness); int < 0 = auto, 0..65536 = fixed */
#define VIDIOCPWCSCONTOUR	_IOW('v', 206, int)
#define VIDIOCPWCGCONTOUR	_IOR('v', 206, int)

  /* Backlight compensation; 0 = off, otherwise on */
#define VIDIOCPWCSBACKLIGHT	_IOW('v', 207, int)
#define VIDIOCPWCGBACKLIGHT	_IOR('v', 207, int)

  /* Flickerless mode; = 0 off, otherwise on */
#define VIDIOCPWCSFLICKER	_IOW('v', 208, int)
#define VIDIOCPWCGFLICKER	_IOR('v', 208, int)  

  /* Dynamic noise reduction; 0 off, 3 = high noise reduction */
#define VIDIOCPWCSDYNNOISE	_IOW('v', 209, int)
#define VIDIOCPWCGDYNNOISE	_IOR('v', 209, int)

// additions to WinPort by BM20031112
/* Private ioctl()s; see pwc-ioctl.h */
extern int wct_ioctl(struct wct_device *pdev, unsigned int cmd, void *arg);

#endif
