/*  -- wctweak_lib project (C)2003 M. Burri 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/*
  This is part of the wctweak_lib project to build a static Win32 library which
  makes heavy use of:
  - the Linux PWC 8.6 webcam driver provided by Nemosoft Unv. (webcam@smcc.demon.nl)
  - the libusb-win32 http://www.sourceforge.net/projects/libusb-win32/ 

  I've renamed the public interface from pwc_ to wct_ as it does not provide
  the full wet of pwc 8.6 and shoul not be mixed up with it.

  Thank you guys for the great work done.

*/

/*
  This file wctweak_lib.h is the public library header file of wctweak_lib build project
*/

/* Version block */
#define WCT_MAJOR	0
#define WCT_MINOR	1
#define WCT_VERSION "0.1"
#define WCT_NAME 	"wctweak_lib"  //BM


extern int wct_dbg_flg;

#ifdef __cplusplus
extern "C" {
#endif


	
/* wct_whitebalance.mode values */
enum wct_wb_mode
{
	WCT_WB_INDOOR		= 0,
	WCT_WB_OUTDOOR		= 1,
	WCT_WB_FL			= 2,
	WCT_WB_MANUAL		= 3,
	WCT_WB_AUTO			= 4
};


enum wct_auto_mode
{
	WCT_AUTO	=	0,
	WCT_FIXED	=	0xff
};

enum wct_onoff_mode
{
	WCT_OFF	=	0,
	WCT_ON	=	0xff
};

enum wct_dynnoise_mode
{
	WCT_DYNN_OFF	=	0,
	WCT_DYYN_LOW	=	1,
	WCT_DYYN_MID	=	2,
	WCT_DYYN_FULL	=	3
};



	
typedef  void *WCT_HANDLE;

/** functions in pwc-if.c */

extern WCT_HANDLE wct_create_handle();
extern int wct_open(WCT_HANDLE wct, unsigned short vendor, unsigned short product);
extern void wct_close(WCT_HANDLE wct);

// returns a string with the USB devices found
// allocates an amount of memory, make sure YOU free it after use
// returns NULL in case of an error
extern char *wct_scanusb(WCT_HANDLE wct);

/** Functions in pwc-ctrl.c */
/* Request a certain video mode. Returns < 0 if not possible */
extern int wct_set_video_mode(struct wct_device *pdev, int width, int height, int frames, int compression, int snapshot);

/* Various controls; should be obvious. Value 0..65535, or < 0 on error */
extern int wct_get_brightness(WCT_HANDLE wct);
extern int wct_set_brightness(WCT_HANDLE wct, int value);

extern int wct_get_contrast(WCT_HANDLE wct);
extern int wct_set_contrast(WCT_HANDLE wct, int value);

extern int wct_get_gamma(WCT_HANDLE wct);
extern int wct_set_gamma(WCT_HANDLE wct, int value);

extern int wct_get_saturation(WCT_HANDLE wct);
extern int wct_set_saturation(WCT_HANDLE wct, int value);


extern int wct_set_agc(struct wct_device *pdev, enum wct_auto_mode mode, int value);
extern int wct_get_agc(struct wct_device *pdev, int *value);

extern int wct_set_shutter_speed(struct wct_device *pdev, enum wct_auto_mode mode, int value);


// RW NVRAM
extern int wct_restore_user(struct wct_device *pdev);
extern int wct_save_user(struct wct_device *pdev);
extern int wct_restore_factory(struct wct_device *pdev);

// Whitebalance
extern int wct_set_awb(struct wct_device *pdev, enum wct_wb_mode mode);
extern enum wct_wb_mode wct_get_awb(struct wct_device *pdev);
extern int wct_set_red_gain(struct wct_device *pdev, int value);
extern int wct_get_red_gain(struct wct_device *pdev);
extern int wct_set_blue_gain(struct wct_device *pdev, int value);
extern int wct_get_blue_gain(struct wct_device *pdev);
/* The following two functions are different, since they only read the
   internal red/blue gains, which may be different from the manual 
   gains set or read above.
 */   
extern int wct_read_red_gain(struct wct_device *pdev);
extern int wct_read_blue_gain(struct wct_device *pdev);

extern int wct_set_wb_speed(struct wct_device *pdev, int speed);
extern int wct_get_wb_speed(struct wct_device *pdev);
extern int wct_set_wb_delay(struct wct_device *pdev, int delay);
extern int wct_get_wb_delay(struct wct_device *pdev);

// LED interface
/* Led on-time / off-time each has a range = 0..25000 */
extern int wct_set_leds(WCT_HANDLE wct, int on_value, int off_value);
extern int wct_get_leds(WCT_HANDLE wct, int *on_value, int *off_value);

// Contour setting (negative number will switch to Auto mode)
extern int wct_set_contour(struct wct_device *pdev, int contour);
extern int wct_get_contour(struct wct_device *pdev, int *contour);

extern int wct_set_backlight(struct wct_device *pdev, enum wct_onoff_mode backlight);
extern enum wct_onoff_mode wct_get_backlight(struct wct_device *pdev);

extern int wct_set_flicker(struct wct_device *pdev, enum wct_onoff_mode flicker);
extern enum wct_onoff_mode wct_get_flicker(struct wct_device *pdev);

extern int wct_set_dynamic_noise(struct wct_device *pdev, enum wct_dynnoise_mode noise);
extern enum wct_dynnoise_mode wct_get_dynamic_noise(struct wct_device *pdev);

extern int wct_get_cmos_sensor(WCT_HANDLE wct);

/* Power down or up the camera; not supported by all models */
extern int wct_camera_power(WCT_HANDLE wct, int power);


/* Private ioctl()s; see pwc-ioctl.h */
extern int wct_ioctl(WCT_HANDLE wct, unsigned int cmd, void *arg);

extern int wct_set_colourmode(struct wct_device *pdev, enum wct_onoff_mode colourmode);
extern enum wct_onoff_mode wct_get_colourmode(struct wct_device *pdev);

#ifdef __cplusplus
}
#endif

